// Fill out your copyright notice in the Description page of Project Settings.


#include "Character/KOFBaseCharacter.h"

#include "Camera/CameraComponent.h"
#include "Components/CapsuleComponent.h"
#include "GameFramework/CharacterMovementComponent.h"
#include "GameFramework/SpringArmComponent.h"
#include "PaperFlipbookComponent.h"

DEFINE_LOG_CATEGORY_STATIC(SideScrollerCharacter, Log, All);

//////////////////////////////////////////////////////////////////////////
// ASideScroller2DCharacter

AKOFBaseCharacter::AKOFBaseCharacter()
{
	// Use only Yaw from the controller and ignore the rest of the rotation.
	bUseControllerRotationPitch = false;
	bUseControllerRotationYaw = true;
	bUseControllerRotationRoll = false;

	// Set the size of our collision capsule.
	GetCapsuleComponent()->SetCapsuleHalfHeight(96.0f);
	GetCapsuleComponent()->SetCapsuleRadius(40.0f);
	
	GetCharacterMovement()->bOrientRotationToMovement = false;

	// Configure character movement
	GetCharacterMovement()->GravityScale = 2.0f;
	GetCharacterMovement()->AirControl = 0.80f;
	GetCharacterMovement()->JumpZVelocity = 1000.f;
	GetCharacterMovement()->GroundFriction = 3.0f;
	GetCharacterMovement()->MaxWalkSpeed = 600.0f;
	GetCharacterMovement()->MaxFlySpeed = 600.0f;

	// Lock character motion onto the XZ plane, so the character can't move in or out of the screen
	GetCharacterMovement()->bConstrainToPlane = true;
	GetCharacterMovement()->SetPlaneConstraintNormal(FVector(1.0, 0.0f, 0.0f));

	// Behave like a traditional 2D platformer character, with a flat bottom instead of a curved capsule bottom
	// Note: This can cause a little floating when going up inclines; you can choose the tradeoff between better
	// behavior on the edge of a ledge versus inclines by setting this to true or false
	GetCharacterMovement()->bUseFlatBaseForFloorChecks = true;

    // 	TextComponent = CreateDefaultSubobject<UTextRenderComponent>(TEXT("IncarGear"));
    // 	TextComponent->SetRelativeScale3D(FVector(3.0f, 3.0f, 3.0f));
    // 	TextComponent->SetRelativeLocation(FVector(35.0f, 5.0f, 20.0f));
    // 	TextComponent->SetRelativeRotation(FRotator(0.0f, 90.0f, 0.0f));
    // 	TextComponent->SetupAttachment(RootComponent);

	// Enable replication on the Sprite component so animations show up when networked
	GetSprite()->SetIsReplicated(true);
	GetSprite()->SetRelativeLocation(FVector(0.0f, 0.0f, -96.0f));
	bReplicates = true;

	// Try to create the sprite component
	Shadow = CreateOptionalDefaultSubobject<UPaperFlipbookComponent>(TEXT("Shadow"));
	if (Shadow)
	{
		Shadow->AlwaysLoadOnClient = true;
		Shadow->AlwaysLoadOnServer = true;
		Shadow->bOwnerNoSee = false;
		Shadow->bAffectDynamicIndirectLighting = true;
		Shadow->PrimaryComponentTick.TickGroup = TG_PrePhysics;
		Shadow->SetupAttachment(GetCapsuleComponent());
		Shadow->SetRelativeLocation(FVector(0.0f, 0.0f, -96.0f));
		Shadow->SetRelativeScale3D(FVector(1.0f, 1.0f, -0.3f));
		Shadow->SetSpriteColor(FLinearColor::Black);
		Shadow->SetTranslucentSortPriority(1000);
	}
}

//////////////////////////////////////////////////////////////////////////
// Animation

void AKOFBaseCharacter::UpdateAnimation()
{
	const FVector PlayerVelocity = GetVelocity();
	const float PlayerSpeedSqr = PlayerVelocity.SizeSquared();
	const float TravelDirection = PlayerVelocity.Y;
	
	// Are we moving or standing still?
	UPaperFlipbook* DesiredAnimation = IdleAnimation;
	if(PlayerSpeedSqr > 0.0f)
	{
		if (TravelDirection > 0.0f)
		{
			DesiredAnimation = WalkFwdAnimation;
		}
		else if (TravelDirection < 0.0f)
		{
			DesiredAnimation = WalkBackAnimation;
		}
	}
	
	if( GetSprite()->GetFlipbook() != DesiredAnimation 	)
	{
		GetSprite()->SetFlipbook(DesiredAnimation);
		Shadow->SetFlipbook(DesiredAnimation);
	}
}

void AKOFBaseCharacter::Tick(float DeltaSeconds)
{
	Super::Tick(DeltaSeconds);
	
	UpdateCharacter();	
}

void AKOFBaseCharacter::BeginPlay()
{
	Super::BeginPlay();

	if(Controller)
	{
		Controller->SetControlRotation(FRotator(0.0f, -90.0f, 0.0));
	}
}

void AKOFBaseCharacter::PostInitializeComponents()
{
	Super::PostInitializeComponents();

	if (!IsPendingKill())
	{
		if (Shadow)
		{
			// force animation tick after movement component updates
			if (Shadow->PrimaryComponentTick.bCanEverTick && GetCharacterMovement())
			{
				Shadow->PrimaryComponentTick.AddPrerequisite(GetCharacterMovement(), GetCharacterMovement()->PrimaryComponentTick);
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////
// Input

void AKOFBaseCharacter::SetupPlayerInputComponent(class UInputComponent* PlayerInputComponent)
{
	// Note: the 'Jump' action and the 'MoveRight' axis are bound to actual keys/buttons/sticks in DefaultInput.ini (editable from Project Settings..Input)
	PlayerInputComponent->BindAction("Jump", IE_Pressed, this, &ACharacter::Jump);
	PlayerInputComponent->BindAction("Jump", IE_Released, this, &ACharacter::StopJumping);
	PlayerInputComponent->BindAxis("MoveRight", this, &AKOFBaseCharacter::MoveRight);
}

void AKOFBaseCharacter::MoveRight(float Value)
{
	/*UpdateChar();*/

	// Apply the input to the character motion
	AddMovementInput(FVector(0.0f, 1.0f, 0.0f), Value);
}

void AKOFBaseCharacter::UpdateCharacter()
{
	// Update animation to match the motion
	UpdateAnimation();

	/*
	// Now setup the rotation of the controller based on the direction we are travelling
	const FVector PlayerVelocity = GetVelocity();	
	float TravelDirection = PlayerVelocity.Y;
	// Set the rotation so that the character faces his direction of travel.
	if (Controller != nullptr)
	{
		if (TravelDirection < 0.0f)
		{
			Controller->SetControlRotation(FRotator(0.0, -90.0f, 0.0f));
		}
		else if (TravelDirection > 0.0f)
		{
			Controller->SetControlRotation(FRotator(0.0f, 90.0f, 0.0));
		}
	}
	*/
}